/*************************************************************************/
/*                                                                       */
/*   Program Name :  datatime_edit                                       */
/*                                                                       */
/*      Version:  1.01  2011-02-09                                       */
/*                                                                       */
/*      Copyright (c) 2009-2011  T.Kondo/NICT All Right Reserved         */
/*                                                                       */
/*************************************************************************/
/*
 
 Edit time label in the K5/VSSP and K5/VSSP32 data file.

Ver 0.00 2009-10-14
    1.00 2009-10-15
    1.01 2011-02-09
      scanf("%s",&XX) was changed to scanf("%s",XX) when XX is 
      declared as like "char XX[20]" to avoid warning at Ubuntu 10.10
      scanf(...) was changed to like i=scanf(...) to avoid warning
      at Ubuntu 10.10

   How to run

       datatime_edit filename [HH:MM:SS|offset [outfile]] [options]

       or

       datatime_edit [options]

         where  filename -- data file name
                HH:MM:SS|offset -- time label of 1st data or time offset in sec
                                    (offset is just added to a current time label)
                outfile  -- output file name after time is editted
                              (default : datatime_edit.dat)
             [options]
                -f filename  ---  set data file name
                -o outfile   ---  set output file name
                -s HH:MM:SS|offset  ---  set 1st time label or offset in seconds
                -n  ---  set non-interactive mode (default is interactive mode)


*/
#define PROGRAM_VERSION "Ver. 2011-02-09"

#define DFLT_FNAME "tds.data"              // default data file name
#define DFLT_ONAME "datatime_edit.dat"     // default output file name

#include "libk5.h"      // common header for IP-VLBI utilities
#include <ctype.h>

typedef struct run_param   
{
    char fname[80];   // data file name
    char oname[80];   // output file name
    int newseconds, newtotalday;
    int soffset;
    int ltype;        // given parameter is 0: time offset 1: "HH:MM:SS"
    int iflag;      // 1: intercative mode  0: non-interactive mode
    int tflag;      // 1: 1st time label or offset is already set
                    // 0: not set
} run_param_t;

void parse_tlabel_field(char *cbuf, int *ltype, int *tflag,
                  int *soffset, int *seconds);
void run_param_get(int argc, char* argv[], run_param_t *runp);
int checkheader(ip_vlbi_file_t *K5);
int fwdNsecHere(unsigned char* ibuf, int nsec, int numb, 
                int imax,ip_vlbi_file_t *dat);
void k5_hd_make(ip_vlbi_file_t *K5);
int conv2sec(char *cbuf);

void parse_tlabel_field(char *cbuf, int *ltype, int *tflag,
                  int *soffset, int *seconds)
{
/*
    parse "HH:MM:SS" or "-1234" or "1234" type to
    proper parameters

 [in]
    cbuf   --   "HH:MM:SS" or offset(sec) in characters

 [return]
    ltype  --   0: offset is given   1: "HH:MM:SS" is given
    tflag  --   0: not valid argument 1: OK
    soffset --  given offset in case of ltype=0
    seconds --  seconds in day in case of HH:MM:SS is given (ltype=1)
         
*/
    int i, len, iok;
    int type, flag;
    int soff,secs;

    flag=0;
    type=0;
    soff=0;
    secs=0;
    len=strlen(cbuf);
    if(cbuf[0]=='-'){  // this is to descriminate command
      // check all character is number or not except for "-"
      iok=1;
      for(i=1; i<len; i++){
          if(isdigit(cbuf[i])==0) iok=0;
      }
      if(iok==1){   // this should be negative number i.e., soffset
          type=0;
          flag=1;
          soff=atoi(cbuf);
      }
    } else {
      // check it is accompanied by ":" or not
      for(i=0; i<len; i++){
         if(cbuf[i]==':') type=1; // given parameter is HH:MM:SS
      }
      if(type==1){ // HH:MM:SS
         secs=conv2sec(cbuf);
         //printf("%s %d\n",cbuf,seconds); // debug
      } else {
         // check all character is number or not
         iok=1;
         for(i=0; i<len; i++){
            if(isdigit(cbuf[i])==0) iok=0;
         }
         if(iok==1){   // this should be positive number i.e., soffset
            type=0;
            flag=1;
            soff=atoi(cbuf);
         }
      }
      flag=1;
    }
    *ltype=type;
    *tflag=flag;
    *soffset=soff;
    *seconds=secs;
}

void run_param_get(int argc, char* argv[], run_param_t *runp)
/*
   get run parameters
*/
{
    char cbuf[80];
    int flag;

    // parameter default set 
    strcpy(runp->fname,DFLT_FNAME);      
    strcpy(runp->oname,DFLT_ONAME); 
    runp->newseconds=0;
    runp->newtotalday=0;
    runp->soffset=0;
    runp->ltype=0;  // default is offset parameter
    runp->iflag=1;  // dfault is interactive mode
    runp->tflag=0; 

    flag=0;

    if(argc < 2) goto exithere;
    // 1st, 2nd, and 3rd parameters check (command or not)
    if (argc>=2){
       strcpy(cbuf,*(argv+1));
       if(cbuf[0]!='-') {
           strcpy(runp->fname,cbuf); // data file name
       } else {
           goto bypass;  //parameter starting with command
       }
    }
    if (argc>=3){
          strcpy(cbuf,*(argv+2));
          parse_tlabel_field(cbuf, &runp->ltype, &runp->tflag,
                  &runp->soffset, &runp->newseconds);
          // check 2nd param is command or not for validity of 3rd parameter
          if(cbuf[0]=='-'){
              if(cbuf[1]=='f' || cbuf[1]=='F') flag=1; // this is command
              if(cbuf[1]=='o' || cbuf[1]=='O') flag=1; // this is command
              if(cbuf[1]=='s' || cbuf[1]=='S') flag=1; // this is command
              if(cbuf[1]=='n' || cbuf[1]=='N') flag=1; // this is command
          }
    }
    if (argc>=4){
       strcpy(cbuf,*(argv+3));
       if(cbuf[0]!='-' && flag==0) strcpy(runp->oname,cbuf); // data out file name
    }

bypass:

    /* command analysis */
    if(cmnd_get(argc,argv,"-n",cbuf)==0) runp->iflag=0; // non-interactive mode
    if(cmnd_get(argc,argv,"-f",cbuf)==0){    // data file name set
        strcpy(runp->fname,cbuf); 
    }
    if(cmnd_get(argc,argv,"-o",cbuf)==0){    // out file name set
        strcpy(runp->oname,cbuf);
    }
    if(cmnd_get(argc,argv,"-s",cbuf)==0){    // set 1st time label of offset in sec
        parse_tlabel_field(cbuf, &runp->ltype, &runp->tflag,
                  &runp->soffset, &runp->newseconds);
    }
exithere:;
}


int checkheader(ip_vlbi_file_t *S) // original is in libcor.c
{
/*
     2006-01-03

    Read 1st header to get sampling prameters and 
    to check sampler type either K5/VSSP or K5/VSSP32
    
       if first block is not header block, found header block automatically
       return value > 0 means above condition occurred
                    =-1 means EOF detection (file closed)

*/
      
       int iok, i, n, i32;
       int cdat[2]={1,4};
       int sdat[4]={1,2,4,8};
       int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                     32000,64000,128000,256000,512000,1024000,2048000}; 
       union HEAD_VSSP hd;
       //fpos_t save;
       i32=0;
       iok=-1;

       S->vflag_old=S->vflag;                  // keep current valid flag
       S->dtime_vflag_old=S->dtime_vflag;  //keep time when vflag decision made
       S->vflag=0;

       //S->hd.format=0; // must be initialize before 1st time call!!

       hd.bbuf=S->hd.bhead;

       memset(S->hd.bhead, 0, 32);  // clear S->hd.head32 array with 0
       /* read 32bit data 1st*/
       S->bytesread = fread( S->hd.bhead, sizeof(char), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 ){
           if(S->bytesread==0){
               printf("checkheader: File EOF! (%s)\n",S->fname);
           } else {
               printf("checkheader: File Read Error! (%s)\n",S->fname);
           }
           i=fclose( S->stream ) ;
           S->open_flag=0;
           i32=-1;
           goto exit_end;
       }

loop1:
       /* read 32 bit data and check sync patter by step-by-step */
       S->bytesread = fread( S->hd.bhead+4, sizeof( char ), 4, S->stream );
       if( ferror( S->stream ) || S->bytesread==0 ){
           if(S->bytesread==0){
               printf("checkheader: File EOF! (%s)\n",S->fname);
           } else {
               printf("checkheader: File Read Error! (%s)\n",S->fname);
           }
           i=fclose( S->stream ) ;
           S->open_flag=0;
           i32=-1;
           goto exit_end;
       }
       /*  debug */
       /*
       printf("%02X%02X%02X%02X %02X\n",
           S->hd.bhead[0],S->hd.bhead[1],S->hd.bhead[2],S->hd.bhead[3],S->hd.bhead[7]);
       printf("%02X%02X%02X%02X %02X\n",
           hd.bbuf[0],hd.bbuf[1],hd.bbuf[2],hd.bbuf[3],hd.bbuf[7]);
       printf("%08X %02X\n",hd.hd->syncptn1,hd.hd->syncptn2);
       printf("totalsec0 %d %d\n",hd.hd->totalsec0, S->hd.bhead[5]*256+S->hd.bhead[4]);
       printf("AD SAMPLING %d %d %02X\n",hd.hd->bitwidth,hd.hd->fsample, S->hd.bhead[6]);
       exit(0);
       */
       /* check sync pattern */
       if(S->hd.format==0){
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == 0x8B) S->hd.format=1;// K5/VSSP
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == 0x8C) S->hd.format=2;// K5/VSSP32
          if(S->hd.format !=0){
             iok=0;  // sync detected
             S->hd.psync2=(unsigned char)hd.hd->syncptn2; // save second sync pattern
          }
       } else {
          if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == S->hd.psync2) iok=0;
       }
       if(iok==0){
           S->hd.sfreq=fdat[hd.hd->fsample];
           S->hd.adbit=sdat[hd.hd->bitwidth];
           S->hd.numch=cdat[hd.hd->chmode];
           S->hd.seconds=hd.hd->totalsec1*65536+hd.hd->totalsec0;
           sec2hms(S->hd.seconds,&S->hd.hh,&S->hd.mm,&S->hd.ss);
           if(S->hd.format == 2){ // more header read
                /* 1st read 4 byte to get AUX field size info */
                S->bytesread = fread( S->hd.bhead+8, sizeof( char ), 4, S->stream );
                if( ferror( S->stream ) || S->bytesread==0 ){
                    if(S->bytesread==0){
                        printf("checkheader: File EOF! (%s)\n",S->fname);
                    } else {
                        printf("checkheader: File Read Error! (%s)\n",S->fname);
                    }
                    i=fclose( S->stream ) ;
                    S->open_flag=0;
                    i32=-1;
                    goto exit_end;
                }
                S->hd.aux_size=hd.hd->auxsize;
                S->hd.year=hd.hd->year+2000;
                S->hd.totalday=hd.hd->totalday;
                S->hd.v_major=hd.hd->v_major;
                S->hd.v_minor=hd.hd->v_minor;
                //printf("debug1 %d %d %d\n",S->hd.aux_size,S->hd.year,S->hd.totalday);
                //printf("debug2 %d %d %d\n",S->hd.v_major,S->hd.v_minor,S->bytesread);

                /* read AUX field */
                S->bytesread = fread( S->hd.bhead+12, sizeof( char ), S->hd.aux_size, S->stream );
                if( ferror( S->stream ) || S->bytesread==0 ){
                    if(S->bytesread==0){
                        printf("checkheader: File EOF! (%s)\n",S->fname);
                    } else {
                        printf("checkheader: File Read Error! (%s)\n",S->fname);
                    }
                    i=fclose( S->stream ) ;
                    S->open_flag=0;
                    i32=-1;
                    goto exit_end;
                }
                /* parse AUX field data */
                S->hd.aux_fmt=hd.hd->fmt;
                S->hd.filter =hd.hd->filter;
                if(S->hd.aux_fmt==1 || S->hd.aux_fmt==2){
                    for(n=0; n<8; n++) S->hd.aux_hostname[n]=hd.hd->hostname[n];
                    S->hd.aux_hostname[8]='\0';
                    if(S->hd.aux_fmt==1){
                       for(n=0; n<8; n++) S->hd.aux_station[n]=hd.hd->station[n];
                       S->hd.aux_station[8]='\0';
                       S->hd.aux_sid2[0]=hd.hd->sid2[0]; S->hd.aux_sid2[1]=hd.hd->sid2[1];
                       S->hd.aux_sid2[2]='\0';
                    } else {
                       S->hd.aux_station[0]='\0';
                       S->hd.aux_sid2[0]='\0';
                    }
                } else {
                    S->hd.aux_hostname[0]='\0';
                    S->hd.aux_station[0]='\0';
                    S->hd.aux_sid2[0]='\0';
                }
                //printf("debug3 fmt,filter %d %d\n",S->hd.aux_fmt,S->hd.filter);
                //printf("debug4 host station sid2 (%s) (%s) (%s)\n",S->hd.aux_hostname,S->hd.aux_station,S->hd.aux_sid2);
               
           }
           goto exit_loop;
       } 
       if(i32 ==0) {
           printf("checkheader: First 64bit is not header block!\n");
           printf("checkheader:   Data file name (%s)\n",S->fname);
           printf("checkheader:   searching header.....");
       }
       /* read next 32 bit data */
       for (n=0; n<4; n++) S->hd.bhead[n]=S->hd.bhead[n+4];
       i32++;
       goto loop1;

exit_loop:

       if(i32 <0) return i32;
       if (iok == 0){   // sync detection
          if(i32 !=0) printf("  found after %d bits read!\n",i32*32);
          if(S->hd.format==1)
             printf("checkheader: Header (K5/VSSP) Sync Detected!!\n");
          if(S->hd.format==2)
             printf("checkheader: Header (K5/VSSP32) Sync Detected!!\n");
          printf("checkheader:   File : %s\n",S->fname);
          printf("checkheader:   A/D(bits) %d  CHs %d  SFreq(kHz) %d",
                    S->hd.adbit,S->hd.numch, S->hd.sfreq);
          printf("  Time %02d:%02d:%02d  sec %d\n",
                    S->hd.hh,S->hd.mm,S->hd.ss,S->hd.seconds);
          // following 4 lines are just for keep backword compatibility
          // should be obsoleted 2006-01-03
          S->adbit=S->hd.adbit;
          S->numch=S->hd.numch;
          S->sfreq=S->hd.sfreq;
          S->hh=S->hd.hh; S->mm=S->hd.mm; S->ss=S->hd.ss; S->seconds=S->hd.seconds;
       }
exit_end:
       if(i32 == 0) S->vflag=1;
       S->dtime_vflag=S->hd.seconds;
       //printf("checkheader:   vflag     dtime_vflag     %d %d\n",S->vflag,S->dtime_vflag);
       //printf("checkheader:   vflag_old dtime_vflag_old %d %d\n",S->vflag_old,S->dtime_vflag_old);
       return i32;
}


int fwdNsecHere(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
    //printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
                return -99;
            }
        }
        
        if(checkheader(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}


void k5_hd_make(ip_vlbi_file_t *K5)
{
    /* update time label of K5/VSSP and K5/VSSP32 header */
    int w1;
    unsigned char b,c;
    /* seconds set */
    w1=K5->hd.seconds;
    K5->hd.bhead[4]=(unsigned char)(w1 & 0xFF);
    K5->hd.bhead[5]=(unsigned char)((w1 & 0xFF00) >> 8);
    b=K5->hd.bhead[6];
    c=(unsigned char)((w1 & 0x10000) >> 16);
    b=(b & 0xFE) | (c & 0x1);
    K5->hd.bhead[6]=b;
    /* total day update */
    if (K5->hd.format == 2){ // VSSP32
       w1=K5->hd.totalday;
       K5->hd.bhead[8]=(unsigned char)(w1 & 0xFF);
       b=K5->hd.bhead[9];
       c=(unsigned char)((w1 & 0x100) >> 8);
       b=(b & 0xFE) | (c & 0x1);
       K5->hd.bhead[9]=b;
    }
}

int conv2sec(char *cbuf)
{
    /*
       convert HH:MM:SS to seconds
    */
    char abuf[20];
    char *cset=":";
    char *p=NULL;
    int k,len,hh,mm,ss;
    int seconds;
   
    len=strlen(cbuf);
    if(len <= 0) return 0;
    hh=0;
    mm=0;
    ss=0;
    k=0;
    sprintf(abuf,"%s\0",strtok(cbuf, cset));
    hh=atoi(abuf);
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s\0",p);
        if(k==1) mm=atoi(abuf);
        if(k==2) ss=atoi(abuf);
        k++;
    }
    seconds=hh*3600+mm*60+ss;
    return seconds;
}




int main(int argc, char *argv[])
{
       int sfreq, adbit, numch;
       int nloop;
       int i,imax;
       int idummy; // 2011-02-09
       int smode,usampl;
       unsigned char *ibuf;
       int nbyte=5000;
       int bytesread;
       int bits, bytes, numb;
       int nch;
       double fsampl,tsampl,fact;
       int *idata;
       int hh,mm,ss;
       int nwrite;
       int headbyte;
       char lab2[60], ltime1st[60];
       char labfsampl[10];
       char cbuf[20];
       run_param_t runp;
       ip_vlbi_file_t X;
       FILE *fo;

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ 
          printf("datatime_edit %s\n",PROGRAM_VERSION);
          printf("\n");
          printf("Usage:\n");
          printf("\n");
          printf("  datatime_edit filename [HH:MM:SS|offset [outfile]] [options]\n");
          printf("    or\n");
          printf("  datatime_edit [options]\n");
          printf("\n");
          printf("    where\n");
          printf("       filename ---- Data file name\n");
          printf("                     if 0 is given, %s will be used\n",DFLT_FNAME);
          printf("       HH:MM:SS|soffset --- Start time label or offset\n");
          printf("                            HH:MM:SS --- Start time label\n");
          printf("                            soffset  --- Start time offset in seconds\n");
          printf("       outfile ---- Output data file name\n");
          printf("                    default is %s\n",DFLT_ONAME);
          printf("       [options]\n");
          printf("         -f filename  ---  set data file name\n");
          printf("         -o outfile   ---  set output file name\n");
          printf("         -s HH:MM:SS|offset  ---  set 1st time label or offset in seconds\n");
          printf("         -n  ---  set non-interactive mode (default is interactive mode)\n");
          printf("\n");
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ 
          printf("datatime_edit %s\n",PROGRAM_VERSION);
          exit(0);
       }
       /* run parameters get */
       run_param_get(argc,argv,&runp);
       X.fname=runp.fname;

       /*  debug
       printf("runp.fname       %s\n",runp.fname);
       printf("runp.oname       %s\n",runp.oname);
       printf("runp.newseconds  %d\n",runp.newseconds);
       printf("runp.soffset     %d\n",runp.soffset);
       printf("runp.ltype       %d\n",runp.ltype);
       printf("runp.tflag       %d\n",runp.tflag);
       printf("runp.iflag       %d\n",runp.iflag);
       */

       /* Program Name Display */
       printf("*********************************************************\n");
       printf("*  K5/VSSP DATA TIME LABEL EDIT PROGRAM (datatime_edit) *\n");
       printf("*           %s    by  T.KONDO/NICT         *\n",PROGRAM_VERSION);
       if(runp.iflag==1){
       printf("*                (INTERACTIVE MODE)                     *\n");
       } else {
       printf("*              (NON-INTERACTIVE MODE)                   *\n");
       }
       printf("*********************************************************\n");
       printf("\n");

       /* Data file open */
       if( (X.stream  = fopen( X.fname, "rb" )) == NULL )
       {
           printf("Data File %s   Open Error! \n",runp.fname);
           goto exitend2;
       }

       /* output file create */
       if( (fo  = fopen( runp.oname, "wb" )) == NULL )
       {
          printf("File  %s  Open Error! \n",runp.oname);
          goto exitend3;
       } 

       /* Data header check to get sampling parameters */
       X.hd.format=0; 
       if(checkheader(&X) < 0){
            goto exitend3;
       }

       /* parameter set */
        sfreq=X.hd.sfreq;
        adbit=X.hd.adbit;
        numch=X.hd.numch;
        fsampl=1000.0*(float)sfreq;   // sampling frequency (Hz)
        tsampl=1.0/fsampl;            // sampling period (sec)
        bits=((long)fsampl)*adbit*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X.fsample=(float)fsampl;

        /* Set extract Ch# to 1 in case of 1 ch board*/
        if(numch == 1) nch=1;   


        /* File read buffer size set */
        smode=1;                         // low resolution mode

        if(smode ==1){ 
            usampl=10000; 
        } else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples

        //printf("imax numb  %d %d\n",imax,numb);    //debug


        ibuf = (unsigned char *)malloc(numb);
        idata = (int *)malloc(4*usampl*sizeof(int));

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq == 40){strcpy(labfsampl,"40kHz"); }
        if(sfreq == 100){strcpy(labfsampl,"100kHz"); }
        if(sfreq == 200){strcpy(labfsampl,"200kHz"); }
        if(sfreq == 500){strcpy(labfsampl,"500kHz"); }
        if(sfreq == 1000){strcpy(labfsampl,"1MHz"); }
        if(sfreq == 2000){strcpy(labfsampl,"2MHz"); }
        if(sfreq == 4000){strcpy(labfsampl,"4MHz"); }
        if(sfreq == 8000){strcpy(labfsampl,"8MHz"); }
        if(sfreq == 16000){strcpy(labfsampl,"16MHz"); }
        if(sfreq == 32000){strcpy(labfsampl,"32MHz"); }
        if(sfreq == 64000){strcpy(labfsampl,"64MHz"); }


       /* Run condition display */

       printf("\n");
       printf("  Data File     : %s",runp.fname);
       if(X.hd.format == 1){
          printf(" (K5/VSSP format)\n");
       } else if (X.hd.format == 2){
          printf(" (K5/VSSP32 format)\n");
       } else {
          printf("\n");
       }
       printf("  Output   File : %s\n",runp.oname);
       printf("\n");
       printf("  Sampling Freq : %s\n",labfsampl);
       printf("  A/D bits      : %d\n",adbit);
       printf("  Number of CHs : %d\n",numch);
       printf("  Start Time(UT) (old)    : %02d:%02d:%02d   Sec in Day = %d\n",
           X.hh,X.mm,X.ss,X.seconds);
       runp.newtotalday=X.hd.totalday;

       if(runp.iflag==1){ // ask operator 
askagain:
           if(runp.tflag==1){ // new time is already given
               printf(" Your setting new time label for 1st data is\n");
               if(runp.ltype==0){
                  runp.newseconds=X.seconds+runp.soffset;
                  if(runp.newseconds >= 86400){
                      runp.newseconds-=86400;
                      runp.newtotalday=X.hd.totalday+1;
                  }
               }
               sec2hms(runp.newseconds, &hh,&mm,&ss);
               printf("  Start Time(UT) (given)  : %02d:%02d:%02d   Sec in Day = %d\n",
                       hh,mm,ss,runp.newseconds);
               printf(" Is this OK? Y/N ");
               cbuf[0]='\0';
               //scanf("%s",&cbuf);
               idummy=scanf("%s",cbuf);  // 2011-02-09
               if(cbuf[0]=='N' || cbuf[0]=='n'){ // change start time
tryagain:
                   printf(" Enter new start time label or offset (HH:MM:SS or offset(sec)) -> ");
                   cbuf[0]='\0';
                   //scanf("%s",&cbuf);
                   idummy=scanf("%s",cbuf);  // 2011-02-09
                   parse_tlabel_field(cbuf, &runp.ltype, &runp.tflag,
                       &runp.soffset, &runp.newseconds);
                   if(runp.tflag==0){
                       printf(" Your input is wrong! Try again!\n");
                       goto tryagain;
                   }
                   goto askagain;
               }
           } else {
tryagain2:
               printf(" Enter new start time label or offset (HH:MM:SS or offset(sec)) -> ");
               cbuf[0]='\0';
               //scanf("%s",&cbuf);
               idummy=scanf("%s",cbuf);  // 2011-02-09
               parse_tlabel_field(cbuf, &runp.ltype, &runp.tflag,
                       &runp.soffset, &runp.newseconds);
               if(runp.tflag==0){
                   printf(" Your input is wrong! Try again!\n");
                   goto tryagain2;
               }
               goto askagain;
           }
       }


       if(runp.ltype==0){
          runp.newseconds=X.seconds+runp.soffset;
          if(runp.newseconds >= 86400){
              runp.newseconds-=86400;
              runp.newtotalday=X.hd.totalday+1;
          }
       }
       sec2hms(runp.newseconds, &hh,&mm,&ss);
       printf("  Start Time(UT) (new)    : %02d:%02d:%02d   Sec in Day = %d\n",
               hh,mm,ss,runp.newseconds);

       printf("  Time Label Edit starting....\n");

       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",hh,mm,ss);
       strcpy(lab2,ltime1st);

       /* continue run info display */
       printf("\n");
       printf("  Out File : %s\n",runp.oname);

       /* Data extraction start */
 //      counts=0;
 //      counts2=0;
 //      m=nch-1;
       nloop=0;

       /* header byte number set */
       headbyte=8;
       if(X.hd.format==2){  // K5/VSSP32 format
           headbyte=12+X.hd.aux_size;
       }



       X.hd.seconds=runp.newseconds;
       X.hd.totalday=runp.newtotalday;

       k5_hd_make(&X);
       /* 1st header copy */
       //nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);
       nwrite = fwrite( X.hd.bhead, sizeof( char ), headbyte, fo);


       /* Data read loop start here */

loop1:
        if(nloop >0) {
           //if(nloop >= period) goto exithere;
           if(checkheader(&X) < 0) goto exithere; 
              //printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
              //nwrite = fwrite( X.bit64, sizeof( char ), 8, fo);
              X.hd.seconds=runp.newseconds;
              X.hd.totalday=runp.newtotalday;
              k5_hd_make(&X);
              
              nwrite = fwrite( X.hd.bhead, sizeof( char ), headbyte, fo);
              sec2hms(runp.newseconds, &hh,&mm,&ss);
              printf("                      ");
              printf("time changed %02d:%02d:%02d ==> %02d:%02d:%02d\n",
                  X.hh,X.mm,X.ss,hh,mm,ss);
        }

        for(i=0; i< imax; i++){
             bytesread = fread( ibuf, sizeof( char ), numb, X.stream );
             if(ferror(X.stream) || bytesread==0 ) goto exithere;
             //printf("bytes read 2 %d\n",bytesread); //debug
             nwrite = fwrite( ibuf, sizeof( char ), numb, fo);
        }
        nloop++;
        runp.newseconds++;
        if(runp.newseconds >= 86400){
              runp.newseconds-=86400;
              runp.newtotalday++;
        }
        goto loop1;
exithere:
       printf("\n");
       printf("Data Time Label edit finished.\n");
       i=fclose( fo ) ;
       
       free(ibuf);
       free(idata);

exitend3: 
       if(X.open_flag!=0) i=fclose( X.stream ) ;

 exitend2: 

       return 0;
}
