/*********************************************************************/
/*                                                                   */
/*   Program Name :  timeadjust                                      */
/*                                                                   */
/*      Version:  0.00  2007-02-23                                   */
/*                1.00  2008-02-19  options '--help' and '--version' */
/*                                  wereadded                        */
/*                                                                   */
/*      Copyright (c) 2007-2008 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*

   Timeadjust puts time of K5/VSSP or K5/VSSP32 board forward or back

         by T.Kondo 2007-02-23

     How to run
            timeadjust sec [SYNC\sync]
                where   sec  -- integer second for adjust (sec)
                                 +VE put a clock forward
                                 -VE put a clock back
                        SYNC|sync -- do 1pps sync

 2007-02-23  atart development

*/
#define TIMEADJUST_VERSION "Ver. 2008-02-19"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // library for IP-VLBI sampler board

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int main(int argc, char **argv)
{
    int fd_in,rv;
    unsigned int time,day,sec; 
    unsigned int secold, nsecmax;
    int nsec;
    int  yyyy, mm, dd; 
    int  h, m, s;
    int  y3; // 7 bits year  (0-127)
    int counter;
    int offset;
    int sig;
    int sync;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;
    char logdir[60]; 
    char hostname[20]; 
    char cbuf[80];
    
    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
        printf("\n");
        printf("timeadjust %s\n", TIMEADJUST_VERSION);
#ifdef _VSSP32
        printf("  compiled for K5/VSSP32 unit\n");
#else
        printf("  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
        printf("\n");
        printf("  Usage: timeadjust sec [SYNC|sync]\n");
        printf("          where   sec  -- integer second for adjust (sec)\n");
        printf("                          +VE put a clock forward\n");
        printf("                          -VE put a clock back\n");
        printf("              SYNC|sync -- do 1pps sync too\n\n");
        exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
        printf("\n");
        printf("timeadjust %s\n", TIMEADJUST_VERSION);
#ifdef _VSSP32
        printf("  compiled for K5/VSSP32 unit\n");
#else
        printf("  compiled for K5/VSSP (IP-VLBI) board\n");
#endif
        printf("\n");
        exit(0);
    }
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

    if(argc >= 2){
       nsec = atoi(*(argv + 1 ));
       if(nsec == 0) {
          printf(" nothing to do.\n");
          exit(0);   // nothing to do
       }
    }
    sync=0;
    if(argc >2){
       if(strncmp("SYNC",*(argv + 2 ),4)==0) sync=1;
       if(strncmp("sync",*(argv + 2 ),4)==0) sync=1;
    }
    
    offset=0;

#ifdef _VSSP32
    printf("K5/VSSP32 (IP-VLBI) Board Time Adjust %s\n",TIMEADJUST_VERSION);
    offset=1;
    if(sync==0) offset=0;
#else
    printf("K5/VSSP (IP-VLBI) Board Time Adjust %s\n",TIMEADJUST_VERSION);
#endif
    
    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"timeadjust %s",TIMEADJUST_VERSION);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(DEV_FILE, O_RDONLY); 
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    /* board reference signal check */
    printf("\n");
    if((sig=board_ref_check2(fd_in))==-1) {
       close(fd_in);
       goto end;
    }
    // time get from device
    time=0;
    secold=90000;
    counter=0;
    while(1){
#ifdef _VSSP32
       rv = ioctl(fd_in, TDSIO_GET_YEAR, &y3);
       yyyy=y3+2000;
#endif
       rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
       if (rv == -1) {
          perror("ioctl() time get failed ");
          close(fd_in);
          return -1;
       }
       day=TDS_GET_DAYS(time);
       sec=TDS_GET_SEC(time);
       sec+=offset;    // for sync delay adjust if necessary
       sec+=nsec;
       if (sec >= 86400){
          sec-=86400;
          day+=1;
       } else if (sec <0){
          sec+=86400;
          day-=1;
       }
       
       if (counter > 0 && sec!=secold) {
          sec2hms(sec,&h,&m,&s);
          //printf(" Debug nsec sec  %d\n",nsec);
          //printf("   Set time  => Total day : %5d    Seconds from 0h : %5d \n",day,sec);
          time = TDS_MAKE_TIME(day,sec);
#ifdef _VSSP32
          // year set
          rv = ioctl(fd_in, TDSIO_SET_YEAR, &y3);  
          if (rv == -1) {
              perror("ioctl() year set failed ");
              sprintf(cbuf,"ioctl() year set failed");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
              close(fd_in);
              return -1;
          }
#endif
          //date and seconds set to sampler
          rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
          if (rv == -1) {
              perror("ioctl() time set failed ");
              sprintf(cbuf,"ioctl() time set failed");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
              close(fd_in);
              return -1;
          }
          break;
       }
       secold=sec;
       counter++;
    }
    
    sprintf(cbuf,"timeadjust (time adjusted by %d sec)",nsec);
    printf("%s\n",cbuf);
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");
    
    //sync to 1PPS
    if(sync==1){
       if(sig==0){
           rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
           if (rv == -1) {
               perror("ioctl() 1pps sync failed ");
               sprintf(cbuf,"ioctl() 1pps sync failed");
               if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
               close(fd_in);
               return -1;
           }
           printf("   synchronized to 1pps\n");
           sprintf(cbuf,"timeadjust (synchronized to 1pps)");
           if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");
       }
    }

    time = 0; 
    secold=90000;
    counter=0;

    close(fd_in);
    // log out
    if(sig==0 && sync==1){
       sprintf(cbuf,"timeadjust (time adjust OK)");
    } else {
       sprintf(cbuf,"timeadjust (time adjust OK but no 1PPS sync made)");
    }
    printf("%s\n",cbuf);
    if(LogWrite2(logdir,hostname,0,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");
    

    //get time from device and display while
    printf("\n");
    i=board_time_disp(10);

end:

    printf("\n\n");
    return 0;
}
